@@ -1,43 +1,43 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Klarälvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 Klarälvdalens Datakonsult AB (KDAB).
+** Contact: https://www.qt.io/licensing/
 **
-** This file is part of the Qt Compositor.
+** This file is part of the QtWaylandClient module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:BSD$
-** You may use this file under the terms of the BSD license as follows:
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** "Redistribution and use in source and binary forms, with or without
-** modification, are permitted provided that the following conditions are
-** met:
-**   * Redistributions of source code must retain the above copyright
-**     notice, this list of conditions and the following disclaimer.
-**   * Redistributions in binary form must reproduce the above copyright
-**     notice, this list of conditions and the following disclaimer in
-**     the documentation and/or other materials provided with the
-**     distribution.
-**   * Neither the name of The Qt Company Ltd nor the names of its
-**     contributors may be used to endorse or promote products derived
-**     from this software without specific prior written permission.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-**
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+
 #include "qwaylanddatadevice_p.h"
 
 #include "qwaylanddatadevicemanager_p.h"
@@ -46,6 +46,7 @@
 #include "qwaylanddnd_p.h"
 #include "qwaylandinputdevice_p.h"
 #include "qwaylanddisplay_p.h"
+#include "qwaylandabstractdecoration_p.h"
 
 #include <QtCore/QMimeData>
 #include <QtGui/QGuiApplication>
@@ -55,8 +56,6 @@
 #include <qpa/qplatformdrag.h>
 #include <qpa/qwindowsysteminterface.h>
 
-#include <QDebug>
-
 QT_BEGIN_NAMESPACE
 
 namespace QtWaylandClient {
@@ -109,7 +108,10 @@ void QWaylandDataDevice::startDrag(QMimeData *mimeData, QWaylandWindow *icon)
 {
     m_dragSource.reset(new QWaylandDataSource(m_display->dndSelectionHandler(), mimeData));
     connect(m_dragSource.data(), &QWaylandDataSource::cancelled, this, &QWaylandDataDevice::dragSourceCancelled);
+
     QWaylandWindow *origin = m_display->currentInputDevice()->pointerFocus();
+    if (!origin)
+        origin = m_display->currentInputDevice()->touchFocus();
 
     start_drag(m_dragSource->object(), origin->object(), icon->object(), m_display->currentInputDevice()->serial());
 }
@@ -128,8 +130,6 @@ void QWaylandDataDevice::data_device_drop()
 {
     QDrag *drag = static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->currentDrag();
 
-    qDebug() << Q_FUNC_INFO << drag << m_dragOffer.data();
-
     QMimeData *dragData = 0;
     Qt::DropActions supportedActions;
     if (drag) {
@@ -153,33 +153,31 @@ void QWaylandDataDevice::data_device_enter(uint32_t serial, wl_surface *surface,
 {
     m_enterSerial = serial;
     m_dragWindow = QWaylandWindow::fromWlSurface(surface)->window();
-    m_dragPoint = QPoint(wl_fixed_to_int(x), wl_fixed_to_int(y));
+    m_dragPoint = calculateDragPosition(x, y, m_dragWindow);
 
-    QDrag *drag = static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->currentDrag();
-
-    QMimeData *dragData;
+    QMimeData *dragData = Q_NULLPTR;
     Qt::DropActions supportedActions;
+
+    m_dragOffer.reset(static_cast<QWaylandDataOffer *>(wl_data_offer_get_user_data(id)));
+    QDrag *drag = static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->currentDrag();
     if (drag) {
         dragData = drag->mimeData();
         supportedActions = drag->supportedActions();
-    } else {
-        m_dragOffer.reset(static_cast<QWaylandDataOffer *>(wl_data_offer_get_user_data(id)));
-        if (m_dragOffer) {
-            dragData = m_dragOffer->mimeData();
-            supportedActions = Qt::CopyAction | Qt::MoveAction | Qt::LinkAction;
-        }
+    } else if (m_dragOffer) {
+        dragData = m_dragOffer->mimeData();
+        supportedActions = Qt::CopyAction | Qt::MoveAction | Qt::LinkAction;
     }
 
     const QPlatformDragQtResponse &response = QWindowSystemInterface::handleDrag(m_dragWindow, dragData, m_dragPoint, supportedActions);
 
     if (drag) {
         static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->setResponse(response);
+    }
+
+    if (response.isAccepted()) {
+        wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, m_dragOffer->firstFormat().toUtf8().constData());
     } else {
-        if (response.isAccepted()) {
-            wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, m_dragOffer->firstFormat().toUtf8().constData());
-        } else {
-            wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, 0);
-        }
+        wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, 0);
     }
 }
 
@@ -202,7 +200,7 @@ void QWaylandDataDevice::data_device_motion(uint32_t time, wl_fixed_t x, wl_fixe
     if (!drag && !m_dragOffer)
         return;
 
-    m_dragPoint = QPoint(wl_fixed_to_int(x), wl_fixed_to_int(y));
+    m_dragPoint = calculateDragPosition(x, y, m_dragWindow);
 
     QMimeData *dragData;
     Qt::DropActions supportedActions;
@@ -218,12 +216,12 @@ void QWaylandDataDevice::data_device_motion(uint32_t time, wl_fixed_t x, wl_fixe
 
     if (drag) {
         static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->setResponse(response);
+    }
+
+    if (response.isAccepted()) {
+        wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, m_dragOffer->firstFormat().toUtf8().constData());
     } else {
-        if (response.isAccepted()) {
-            wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, m_dragOffer->firstFormat().toUtf8().constData());
-        } else {
-            wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, 0);
-        }
+        wl_data_offer_accept(m_dragOffer->object(), m_enterSerial, 0);
     }
 }
 
@@ -246,7 +244,6 @@ void QWaylandDataDevice::selectionSourceCancelled()
 void QWaylandDataDevice::dragSourceCancelled()
 {
     m_dragSource.reset();
-
 }
 
 void QWaylandDataDevice::dragSourceTargetChanged(const QString &mimeType)
@@ -254,6 +251,19 @@ void QWaylandDataDevice::dragSourceTargetChanged(const QString &mimeType)
     static_cast<QWaylandDrag *>(QGuiApplicationPrivate::platformIntegration()->drag())->updateTarget(mimeType);
 }
 
+QPoint QWaylandDataDevice::calculateDragPosition(int x, int y, QWindow *wnd) const
+{
+    QPoint pnt(wl_fixed_to_int(x), wl_fixed_to_int(y));
+    if (wnd) {
+        QWaylandWindow *wwnd = static_cast<QWaylandWindow*>(m_dragWindow->handle());
+        if (wwnd && wwnd->decoration()) {
+            pnt -= QPoint(wwnd->decoration()->margins().left(),
+                          wwnd->decoration()->margins().top());
+        }
+    }
+    return pnt;
+}
+
 }
 
 QT_END_NAMESPACE
