@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,10 +49,13 @@
 #include "qwaylandcursor_p.h"
 #include "qwaylanddisplay_p.h"
 #include "qwaylandshmbackingstore_p.h"
-#include "../shared/qwaylandxkb.h"
+#include "../shared/qwaylandxkb_p.h"
+#include "qwaylandinputcontext_p.h"
 
 #include <QtGui/private/qpixmap_raster_p.h>
+#include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformwindow.h>
+#include <qpa/qplatforminputcontext.h>
 #include <QDebug>
 
 #include <unistd.h>
@@ -69,7 +78,6 @@ QWaylandInputDevice::Keyboard::Keyboard(QWaylandInputDevice *p)
     , mXkbMap(0)
     , mXkbState(0)
 #endif
-    , mFocusCallback(0)
     , mNativeModifiers(0)
 {
     connect(&mRepeatTimer, SIGNAL(timeout()), this, SLOT(repeatKey()));
@@ -122,8 +130,6 @@ QWaylandInputDevice::Keyboard::~Keyboard()
 #endif
     if (mFocus)
         QWindowSystemInterface::handleWindowActivated(0);
-    if (mFocusCallback)
-        wl_callback_destroy(mFocusCallback);
     if (mParent->mVersion >= 3)
         wl_keyboard_release(object());
     else
@@ -168,15 +174,16 @@ QWaylandInputDevice::Touch::~Touch()
 
 QWaylandInputDevice::QWaylandInputDevice(QWaylandDisplay *display, int version, uint32_t id)
     : QObject()
-    , QtWayland::wl_seat(display->wl_registry(), id, qMin(version, 3))
+    , QtWayland::wl_seat(display->wl_registry(), id, qMin(version, 4))
     , mQDisplay(display)
     , mDisplay(display->wl_display())
-    , mVersion(qMin(version, 3))
+    , mVersion(qMin(version, 4))
     , mCaps(0)
     , mDataDevice(0)
     , mKeyboard(0)
     , mPointer(0)
     , mTouch(0)
+    , mTextInput(0)
     , mTime(0)
     , mSerial(0)
     , mTouchDevice(0)
@@ -185,6 +192,9 @@ QWaylandInputDevice::QWaylandInputDevice(QWaylandDisplay *display, int version,
         mDataDevice = mQDisplay->dndSelectionHandler()->getDataDevice(this);
     }
 
+    if (mQDisplay->textInputManager()) {
+        mTextInput = new QWaylandTextInput(mQDisplay, mQDisplay->textInputManager()->get_text_input(wl_seat()));
+    }
 }
 
 QWaylandInputDevice::~QWaylandInputDevice()
@@ -269,6 +279,16 @@ QWaylandDataDevice *QWaylandInputDevice::dataDevice() const
     return mDataDevice;
 }
 
+void QWaylandInputDevice::setTextInput(QWaylandTextInput *textInput)
+{
+    mTextInput = textInput;
+}
+
+QWaylandTextInput *QWaylandInputDevice::textInput() const
+{
+    return mTextInput;
+}
+
 void QWaylandInputDevice::removeMouseButtonFromState(Qt::MouseButton button)
 {
     if (mPointer)
@@ -591,10 +611,7 @@ void QWaylandInputDevice::Keyboard::keyboard_enter(uint32_t time, struct wl_surf
     QWaylandWindow *window = QWaylandWindow::fromWlSurface(surface);
     mFocus = window;
 
-    if (!mFocusCallback) {
-        mFocusCallback = wl_display_sync(mParent->mDisplay);
-        wl_callback_add_listener(mFocusCallback, &QWaylandInputDevice::Keyboard::callback, this);
-    }
+    mParent->mQDisplay->handleKeyboardFocusChanged(mParent);
 }
 
 void QWaylandInputDevice::Keyboard::keyboard_leave(uint32_t time, struct wl_surface *surface)
@@ -609,32 +626,29 @@ void QWaylandInputDevice::Keyboard::keyboard_leave(uint32_t time, struct wl_surf
 
     mFocus = NULL;
 
-    // Use a callback to set the focus because we may get a leave/enter pair, and
-    // the latter one would be lost in the QWindowSystemInterface queue, if
-    // we issue the handleWindowActivated() calls immediately.
-    if (!mFocusCallback) {
-        mFocusCallback = wl_display_sync(mParent->mDisplay);
-        wl_callback_add_listener(mFocusCallback, &QWaylandInputDevice::Keyboard::callback, this);
-    }
+    mParent->mQDisplay->handleKeyboardFocusChanged(mParent);
+
     mRepeatTimer.stop();
 }
 
-const wl_callback_listener QWaylandInputDevice::Keyboard::callback = {
-    QWaylandInputDevice::Keyboard::focusCallback
-};
-
-void QWaylandInputDevice::Keyboard::focusCallback(void *data, struct wl_callback *callback, uint32_t time)
+static void sendKey(QWindow *tlw, ulong timestamp, QEvent::Type type, int key, Qt::KeyboardModifiers modifiers,
+                    quint32 nativeScanCode, quint32 nativeVirtualKey, quint32 nativeModifiers,
+                    const QString& text = QString(), bool autorep = false, ushort count = 1)
 {
-    Q_UNUSED(time);
-    Q_UNUSED(callback);
-    QWaylandInputDevice::Keyboard *self = static_cast<QWaylandInputDevice::Keyboard *>(data);
-    if (self->mFocusCallback) {
-        wl_callback_destroy(self->mFocusCallback);
-        self->mFocusCallback = 0;
+    QPlatformInputContext *inputContext = QGuiApplicationPrivate::platformIntegration()->inputContext();
+    bool filtered = false;
+
+    if (inputContext) {
+        QKeyEvent event(type, key, modifiers, nativeScanCode, nativeVirtualKey, nativeModifiers,
+                        text, autorep, count);
+        event.setTimestamp(timestamp);
+        filtered = inputContext->filterEvent(&event);
     }
 
-    self->mParent->mQDisplay->setLastKeyboardFocusInputDevice(self->mFocus ? self->mParent : 0);
-    QWindowSystemInterface::handleWindowActivated(self->mFocus ? self->mFocus->window() : 0);
+    if (!filtered) {
+        QWindowSystemInterface::handleExtendedKeyEvent(tlw, timestamp, type, key, modifiers,
+                nativeScanCode, nativeVirtualKey, nativeModifiers, text, autorep, count);
+    }
 }
 
 void QWaylandInputDevice::Keyboard::keyboard_key(uint32_t serial, uint32_t time, uint32_t key, uint32_t state)
@@ -665,30 +679,12 @@ void QWaylandInputDevice::Keyboard::keyboard_key(uint32_t serial, uint32_t time,
 
     Qt::KeyboardModifiers modifiers = mParent->modifiers();
 
-    uint utf32 = xkb_keysym_to_utf32(sym);
-    if (utf32)
-        text = QString::fromUcs4(&utf32, 1);
+    std::tie(qtkey, text) = QWaylandXkb::keysymToQtKey(sym, modifiers);
 
-    qtkey = QWaylandXkb::keysymToQtKey(sym, modifiers, text);
-
-
-    // Map control + letter to proper text
-    if (utf32 >= 'A' && utf32 <= '~' && (modifiers & Qt::ControlModifier)) {
-        utf32 &= ~0x60;
-        text = QString::fromUcs4(&utf32, 1);
-    }
-
-    QWindowSystemInterface::handleExtendedKeyEvent(window->window(),
-                                                    time, type, qtkey,
-                                                    modifiers,
-                                                    code, sym, mNativeModifiers, text);
+    sendKey(window->window(), time, type, qtkey, modifiers, code, sym, mNativeModifiers, text);
 #else
     // Generic fallback for single hard keys: Assume 'key' is a Qt key code.
-    QWindowSystemInterface::handleExtendedKeyEvent(window->window(),
-                                                    time, type,
-                                                    qtkey,
-                                                    Qt::NoModifier,
-                                                    code, 0, 0);
+    sendKey(window->window(), time, type, qtkey, Qt::NoModifier, code, 0, 0);
 #endif
 
     if (state == WL_KEYBOARD_KEY_STATE_PRESSED
@@ -713,27 +709,21 @@ void QWaylandInputDevice::Keyboard::keyboard_key(uint32_t serial, uint32_t time,
 void QWaylandInputDevice::Keyboard::repeatKey()
 {
     mRepeatTimer.setInterval(25);
-    QWindowSystemInterface::handleExtendedKeyEvent(mFocus->window(),
-                                                   mRepeatTime, QEvent::KeyRelease, mRepeatKey,
-                                                   modifiers(),
-                                                   mRepeatCode,
+    sendKey(mFocus->window(), mRepeatTime, QEvent::KeyRelease, mRepeatKey, modifiers(), mRepeatCode,
 #ifndef QT_NO_WAYLAND_XKB
-                                                   mRepeatSym, mNativeModifiers,
+            mRepeatSym, mNativeModifiers,
 #else
-                                                   0, 0,
+            0, 0,
 #endif
-                                                   mRepeatText, true);
+            mRepeatText, true);
 
-    QWindowSystemInterface::handleExtendedKeyEvent(mFocus->window(),
-                                                   mRepeatTime, QEvent::KeyPress, mRepeatKey,
-                                                   modifiers(),
-                                                   mRepeatCode,
+    sendKey(mFocus->window(), mRepeatTime, QEvent::KeyPress, mRepeatKey, modifiers(), mRepeatCode,
 #ifndef QT_NO_WAYLAND_XKB
-                                                   mRepeatSym, mNativeModifiers,
+            mRepeatSym, mNativeModifiers,
 #else
-                                                   0, 0,
+            0, 0,
 #endif
-                                                   mRepeatText, true);
+            mRepeatText, true);
 }
 
 void QWaylandInputDevice::Keyboard::keyboard_modifiers(uint32_t serial,
