@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the config.tests of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -50,10 +56,12 @@
 
 #include <QtCore/QFileInfo>
 #include <QtCore/QPointer>
+#include <QtCore/QRegularExpression>
 #include <QtGui/QWindow>
 
 #include <QGuiApplication>
 #include <qpa/qwindowsysteminterface.h>
+#include <QtGui/private/qwindow_p.h>
 
 #include <QtCore/QDebug>
 
@@ -84,6 +92,7 @@ QWaylandWindow::QWaylandWindow(QWindow *window)
     , mState(Qt::WindowNoState)
     , mMask()
     , mBackingStore(Q_NULLPTR)
+    , mUpdateRequested(false)
 {
     static WId id = 1;
     mWindowId = id++;
@@ -131,22 +140,32 @@ void QWaylandWindow::initWindow()
         mShellSurface->setTitle(window()->title());
 
         // The appId is the desktop entry identifier that should follow the
-        // reverse DNS convention (see http://standards.freedesktop.org/desktop-entry-spec/latest/ar01s02.html),
-        // use the application domain if available, otherwise the executable base name.
-        // According to xdg-shell the appId is only the name, without the .desktop suffix.
-        QFileInfo fi = QCoreApplication::instance()->applicationFilePath();
-        QStringList domainName =
-            QCoreApplication::instance()->organizationDomain().split(QLatin1Char('.'),
-                                                                     QString::SkipEmptyParts);
-
-        if (domainName.isEmpty()) {
-            mShellSurface->setAppId(fi.baseName());
+        // reverse DNS convention (see http://standards.freedesktop.org/desktop-entry-spec/latest/ar01s02.html).
+        // According to xdg-shell the appId is only the name, without
+        // the .desktop suffix.
+        //
+        // If the application specifies the desktop file name use that
+        // removing the ".desktop" suffix, otherwise fall back to the
+        // executable name and prepend the reversed organization domain
+        // when available.
+        if (!QGuiApplication::desktopFileName().isEmpty()) {
+            QString name = QGuiApplication::desktopFileName();
+            mShellSurface->setAppId(name.replace(QRegularExpression(QLatin1String("\\.desktop$")), QString()));
         } else {
-            QString appId;
-            for (int i = 0; i < domainName.count(); ++i)
-                appId.prepend(QLatin1Char('.')).prepend(domainName.at(i));
-            appId.append(fi.baseName());
-            mShellSurface->setAppId(appId);
+            QFileInfo fi = QCoreApplication::instance()->applicationFilePath();
+            QStringList domainName =
+                QCoreApplication::instance()->organizationDomain().split(QLatin1Char('.'),
+                                                                         QString::SkipEmptyParts);
+
+            if (domainName.isEmpty()) {
+                mShellSurface->setAppId(fi.baseName());
+            } else {
+                QString appId;
+                for (int i = 0; i < domainName.count(); ++i)
+                    appId.prepend(QLatin1Char('.')).prepend(domainName.at(i));
+                appId.append(fi.baseName());
+                mShellSurface->setAppId(appId);
+            }
         }
     }
 
@@ -170,7 +189,10 @@ void QWaylandWindow::initWindow()
     if (QScreen *s = window()->screen())
         setOrientationMask(s->orientationUpdateMask());
     setWindowFlags(window()->flags());
-    setGeometry_helper(window()->geometry());
+    if (window()->geometry().isEmpty())
+        setGeometry_helper(QRect(QPoint(), QSize(500,500)));
+    else
+        setGeometry_helper(window()->geometry());
     setMask(window()->mask());
     setWindowStateInternal(window()->windowState());
     handleContentOrientationChange(window()->contentOrientation());
@@ -408,8 +430,8 @@ void QWaylandWindow::setCanResize(bool canResize)
             doResize();
             QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
         } else if (mResizeDirty) {
-            QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
             mResizeDirty = false;
+            QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
         }
     }
 }
@@ -430,10 +452,20 @@ void QWaylandWindow::requestResize()
 
 void QWaylandWindow::attach(QWaylandBuffer *buffer, int x, int y)
 {
-    if (buffer)
+    if (mFrameCallback) {
+        wl_callback_destroy(mFrameCallback);
+        mFrameCallback = 0;
+    }
+
+    if (buffer) {
+        mFrameCallback = frame();
+        wl_callback_add_listener(mFrameCallback, &QWaylandWindow::callbackListener, this);
+        mWaitingForFrameSync = true;
+
         attach(buffer->buffer(), x, y);
-    else
+    } else {
         QtWayland::wl_surface::attach(0, 0, 0);
+    }
 }
 
 void QWaylandWindow::attachOffset(QWaylandBuffer *buffer)
@@ -444,13 +476,6 @@ void QWaylandWindow::attachOffset(QWaylandBuffer *buffer)
 
 void QWaylandWindow::damage(const QRect &rect)
 {
-    //We have to do sync stuff before calling damage, or we might
-    //get a frame callback before we get the timestamp
-    if (!mWaitingForFrameSync) {
-        mFrameCallback = frame();
-        wl_callback_add_listener(mFrameCallback,&QWaylandWindow::callbackListener,this);
-        mWaitingForFrameSync = true;
-    }
     damage(rect.x(), rect.y(), rect.width(), rect.height());
 }
 
@@ -462,12 +487,14 @@ void QWaylandWindow::frameCallback(void *data, struct wl_callback *callback, uin
 {
     Q_UNUSED(time);
     QWaylandWindow *self = static_cast<QWaylandWindow*>(data);
-    if (callback != self->mFrameCallback) // might be a callback caused by the shm backingstore
-        return;
+
     self->mWaitingForFrameSync = false;
-    if (self->mFrameCallback) {
-        wl_callback_destroy(self->mFrameCallback);
-        self->mFrameCallback = 0;
+    wl_callback_destroy(callback);
+    self->mFrameCallback = 0;
+    if (self->mUpdateRequested) {
+        QWindowPrivate *w = QWindowPrivate::get(self->window());
+        w->deliverUpdateRequest();
+        self->mUpdateRequested = false;
     }
 }
 
@@ -500,6 +527,11 @@ QWaylandSubSurface *QWaylandWindow::subSurfaceWindow() const
     return mSubSurfaceWindow;
 }
 
+bool QWaylandWindow::shellManagesActiveState() const
+{
+    return mShellSurface && mShellSurface->shellManagesActiveState();
+}
+
 void QWaylandWindow::handleContentOrientationChange(Qt::ScreenOrientation orientation)
 {
     if (mDisplay->compositorVersion() < 2)
@@ -818,6 +850,7 @@ bool QWaylandWindow::setWindowStateInternal(Qt::WindowState state)
     mState = state;
 
     if (mShellSurface) {
+        createDecoration();
         switch (state) {
             case Qt::WindowFullScreen:
                 mShellSurface->setFullscreen();
@@ -870,6 +903,19 @@ QVariant QWaylandWindow::property(const QString &name, const QVariant &defaultVa
     return m_properties.value(name, defaultValue);
 }
 
+void QWaylandWindow::requestUpdate()
+{
+    if (!mFrameCallback)
+        QPlatformWindow::requestUpdate();
+    else
+        mUpdateRequested = true;
+}
+
+void QWaylandWindow::addAttachOffset(const QPoint point)
+{
+    mOffset += point;
+}
+
 }
 
 QT_END_NAMESPACE
