@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -196,8 +202,13 @@ QByteArray waylandToCType(const QByteArray &waylandType, const QByteArray &inter
         return "int32_t";
     else if (waylandType == "array")
         return "wl_array *";
-    else if (waylandType == "object" || waylandType == "new_id")
-        return isServerSide() ? "struct ::wl_resource *" : interface.isEmpty() ? "struct ::wl_object *" : "struct ::" + interface + " *";
+    else if (waylandType == "object" || waylandType == "new_id") {
+        if (isServerSide())
+            return "struct ::wl_resource *";
+        if (interface.isEmpty())
+            return "struct ::wl_object *";
+        return "struct ::" + interface + " *";
+    }
     return waylandType;
 }
 
@@ -381,7 +392,6 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
 
         printf("\n");
         printf("QT_BEGIN_NAMESPACE\n");
-
         QByteArray serverExport;
         if (headerPath.size()) {
             serverExport = QByteArray("Q_WAYLAND_SERVER_") + preProcessorProtocolName + "_EXPORT";
@@ -412,6 +422,7 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("    public:\n");
             printf("        %s(struct ::wl_client *client, int id, int version);\n", interfaceName);
             printf("        %s(struct ::wl_display *display, int version);\n", interfaceName);
+            printf("        %s(struct ::wl_resource *resource);\n", interfaceName);
             printf("        %s();\n", interfaceName);
             printf("\n");
             printf("        virtual ~%s();\n", interfaceName);
@@ -428,11 +439,12 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("            struct ::wl_client *client() const { return handle->client; }\n");
             printf("            int version() const { return wl_resource_get_version(handle); }\n");
             printf("\n");
-            printf("            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }\n");
+            printf("            static Resource *fromResource(struct ::wl_resource *resource);\n");
             printf("        };\n");
             printf("\n");
             printf("        void init(struct ::wl_client *client, int id, int version);\n");
             printf("        void init(struct ::wl_display *display, int version);\n");
+            printf("        void init(struct ::wl_resource *resource);\n");
             printf("\n");
             printf("        Resource *add(struct ::wl_client *client, int version);\n");
             printf("        Resource *add(struct ::wl_client *client, int id, int version);\n");
@@ -446,6 +458,11 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("\n");
             printf("        bool isGlobal() const { return m_global != 0; }\n");
             printf("        bool isResource() const { return m_resource != 0; }\n");
+            printf("\n");
+            printf("        static const struct ::wl_interface *interface();\n");
+            printf("        static QByteArray interfaceName() { return interface()->name; }\n");
+            printf("        static int interfaceVersion() { return interface()->version; }\n");
+            printf("\n");
 
             printEnums(interface.enums);
 
@@ -487,6 +504,7 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("        static void destroy_func(struct ::wl_resource *client_resource);\n");
             printf("\n");
             printf("        Resource *bind(struct ::wl_client *client, uint32_t id, int version);\n");
+            printf("        Resource *bind(struct ::wl_resource *handle);\n");
 
             if (hasRequests) {
                 printf("\n");
@@ -565,6 +583,15 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("    }\n");
             printf("\n");
 
+            printf("    %s::%s(struct ::wl_resource *resource)\n", interfaceName, interfaceName);
+            printf("        : m_resource_map()\n");
+            printf("        , m_resource(0)\n");
+            printf("        , m_global(0)\n");
+            printf("    {\n");
+            printf("        init(resource);\n");
+            printf("    }\n");
+            printf("\n");
+
             printf("    %s::%s()\n", interfaceName, interfaceName);
             printf("        : m_resource_map()\n");
             printf("        , m_resource(0)\n");
@@ -584,6 +611,12 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("    }\n");
             printf("\n");
 
+            printf("    void %s::init(struct ::wl_resource *resource)\n", interfaceName);
+            printf("    {\n");
+            printf("        m_resource = bind(resource);\n");
+            printf("    }\n");
+            printf("\n");
+
             printf("    %s::Resource *%s::add(struct ::wl_client *client, int version)\n", interfaceName, interfaceName);
             printf("    {\n");
             printf("        Resource *resource = bind(client, 0, version);\n");
@@ -607,6 +640,12 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("    }\n");
             printf("\n");
 
+            printf("    const struct wl_interface *%s::interface()\n", interfaceName);
+            printf("    {\n");
+            printf("        return &::%s_interface;\n", interfaceName);
+            printf("    }\n");
+            printf("\n");
+
             printf("    %s::Resource *%s::%s_allocate()\n", interfaceName, interfaceName, interfaceNameStripped);
             printf("    {\n");
             printf("        return new Resource;\n");
@@ -651,10 +690,17 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             //and use function overloading instead. Jan do you have a lot of code dependent on this behavior?
             printf("    %s::Resource *%s::bind(struct ::wl_client *client, uint32_t id, int version)\n", interfaceName, interfaceName);
             printf("    {\n");
+            printf("        Q_ASSERT_X(!wl_client_get_object(client, id), \"QWaylandObject bind\", QStringLiteral(\"binding to object %%1 more than once\").arg(id).toLocal8Bit().constData());\n");
+            printf("        struct ::wl_resource *handle = wl_resource_create(client, &::%s_interface, version, id);\n", interfaceName);
+            printf("        return bind(handle);\n");
+            printf("    }\n");
+            printf("\n");
+
+            printf("    %s::Resource *%s::bind(struct ::wl_resource *handle)\n", interfaceName, interfaceName);
+            printf("    {\n");
             printf("        Resource *resource = %s_allocate();\n", interfaceNameStripped);
             printf("        resource->%s_object = this;\n", interfaceNameStripped);
             printf("\n");
-            printf("        struct ::wl_resource *handle = wl_resource_create(client, &::%s_interface, version, id);\n", interfaceName);
             printf("        wl_resource_set_implementation(handle, %s, resource, destroy_func);", interfaceMember.constData());
             printf("\n");
             printf("        resource->handle = handle;\n");
@@ -662,6 +708,13 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("        return resource;\n");
             printf("    }\n");
 
+            printf("    %s::Resource *%s::Resource::fromResource(struct ::wl_resource *resource)\n", interfaceName, interfaceName);
+            printf("    {\n");
+            printf("        if (wl_resource_instance_of(resource, &::%s_interface, %s))\n",  interfaceName, interfaceMember.constData());
+            printf("            return static_cast<Resource *>(resource->data);\n");
+            printf("        return 0;\n");
+            printf("    }\n");
+
             if (hasRequests) {
                 printf("\n");
                 printf("    const struct ::%s_interface %s::m_%s_interface = {", interfaceName, interfaceName, interfaceName);
@@ -832,6 +885,8 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("        const struct ::%s *object() const { return m_%s; }\n", interfaceName, interfaceName);
             printf("\n");
             printf("        bool isInitialized() const;\n");
+            printf("\n");
+            printf("        static const struct ::wl_interface *interface();\n");
 
             printEnums(interface.enums);
 
@@ -839,7 +894,14 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
                 printf("\n");
                 foreach (const WaylandEvent &e, interface.requests) {
                     const WaylandArgument *new_id = newIdArgument(e.arguments);
-                    printf("        %s", new_id ? (new_id->interface.isEmpty() ? "void *" : "struct ::" + new_id->interface + " *").constData() : "void ");
+                    QByteArray new_id_str = "void ";
+                    if (new_id) {
+                        if (new_id->interface.isEmpty())
+                            new_id_str = "void *";
+                        else
+                            new_id_str = "struct ::" + new_id->interface + " *";
+                    }
+                    printf("        %s", new_id_str.constData());
                     printEvent(e);
                     printf(";\n");
                 }
@@ -950,12 +1012,25 @@ void process(QXmlStreamReader &xml, const QByteArray &headerPath, const QByteArr
             printf("    {\n");
             printf("        return m_%s != 0;\n", interfaceName);
             printf("    }\n");
+            printf("\n");
+
+            printf("    const struct wl_interface *%s::interface()\n", interfaceName);
+            printf("    {\n");
+            printf("        return &::%s_interface;\n", interfaceName);
+            printf("    }\n");
 
             for (int i = 0; i < interface.requests.size(); ++i) {
                 printf("\n");
                 const WaylandEvent &e = interface.requests.at(i);
                 const WaylandArgument *new_id = newIdArgument(e.arguments);
-                printf("    %s%s::", new_id ? (new_id->interface.isEmpty() ? "void *" : "struct ::" + new_id->interface + " *").constData() : "void ", interfaceName);
+                QByteArray new_id_str = "void ";
+                if (new_id) {
+                    if (new_id->interface.isEmpty())
+                        new_id_str = "void *";
+                    else
+                        new_id_str = "struct ::" + new_id->interface + " *";
+                }
+                printf("    %s%s::", new_id_str.constData(), interfaceName);
                 printEvent(e);
                 printf("\n");
                 printf("    {\n");
